/*
 * Copyright (C) 2000-2001 Damien Douxchamps  <douxchamps@ieee.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "build_menus.h"
#include "update_frames.h"
#include "build_windows.h"
#include "update_windows.h"
#include "definitions.h"
#include "capture.h"
#include "callback_proc.h"
#include <libdc1394/dc1394_control.h>

extern GtkWidget *capture_window;
extern GtkWidget *format7_window;
extern GtkWidget *status_window;
extern GtkWidget *temperature_window;
extern GtkWidget *about_window;
extern GtkWidget *aperture_window;
extern GtkWidget *color_window;
extern GtkWidget *porthole_window;
extern dc1394_camerainfo *camera;
extern dc1394_feature_set *feature_set;
extern dc1394_camerainfo *cameras;
extern dc1394_feature_set *feature_sets;
extern dc1394_miscinfo *misc_info;
extern dc1394_miscinfo *misc_infos;
extern dc1394_cameracapture *capture;
extern dc1394_cameracapture *captures;
extern Format7Info *format7_info;
extern Format7Info *format7_infos;
extern int current_camera;
extern guint gIdleID;
extern porthole_info pi;
extern guint gCaptureIdleID;
extern int overlay_power;


gboolean
on_commander_window_delete_event       (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_exit(0);
  return FALSE;
}


void
on_file_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left almost blank: intermediate menu
}


void
on_load_setup_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dialog = create_file_selector_window();
  gtk_window_set_title (GTK_WINDOW (dialog), "Select a setup file to load...");
  gtk_widget_show(dialog);

  // to be continued...
}


void
on_save_setup_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // TO BE FILLED
}


void
on_save_setup_as_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dialog = create_file_selector_window();
  gtk_window_set_title (GTK_WINDOW (dialog), "Save setup as...");
  gtk_widget_show(dialog);

  // to be continued...
}


void
on_exit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_exit(0);
}


void
on_porthole_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *scope = lookup_widget(porthole_window, "camera_scope");
  gtk_widget_show (porthole_window);

  if (overlay_power==1)
    if (pi.handle == NULL)
      // start a separate thread for iso receive
      if (IsoStartThread(scope) > 0)
	// run a gtk idler thread to update the display
	gIdleID = gtk_idle_add( porthole_idler, NULL);
}

gboolean
on_porthole_window_delete_event       (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_idle_remove(gIdleID);
  IsoStopThread();
  pi.is_open = FALSE;
  gtk_widget_hide( porthole_window);
  return TRUE;
}

void
on_color_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (color_window);
}


void
on_aperture_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (aperture_window);
}


void
on_format_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left almost blank: intermediate menu
}


void
on_format_0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left almost blank: intermediate menu
}



void
on_format_1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left almost blank: intermediate menu
}

void
on_format_2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left almost blank: intermediate menu
}


void
on_format_6_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left almost blank: intermediate menu
}


void
on_format_7_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left almost blank: intermediate menu
}

void
on_help_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left almost blank: intermediate menu
}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  about_window = create_about_window ();
  gtk_widget_show (about_window);
}


gboolean
on_file_selector_window_delete_event   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  GtkWidget *dialog = gtk_widget_get_toplevel(widget);
  gtk_widget_destroy(dialog);
  return FALSE;
}

void
on_f0_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_VGA_NONCOMPRESSED);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_160x120_YUV444);
  misc_info->format=FORMAT_VGA_NONCOMPRESSED;
  misc_info->mode=MODE_160x120_YUV444;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f0_mode1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_VGA_NONCOMPRESSED);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_320x240_YUV422);
  misc_info->format=FORMAT_VGA_NONCOMPRESSED;
  misc_info->mode=MODE_320x240_YUV422;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f0_mode2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_VGA_NONCOMPRESSED);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_640x480_YUV411);
  misc_info->format=FORMAT_VGA_NONCOMPRESSED;
  misc_info->mode=MODE_640x480_YUV411;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f0_mode3_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_VGA_NONCOMPRESSED);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_640x480_YUV422);
  misc_info->format=FORMAT_VGA_NONCOMPRESSED;
  misc_info->mode=MODE_640x480_YUV422;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f0_mode4_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_VGA_NONCOMPRESSED);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_640x480_RGB);
  misc_info->format=FORMAT_VGA_NONCOMPRESSED;
  misc_info->mode=MODE_640x480_RGB;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f0_mode5_ativate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_VGA_NONCOMPRESSED);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_640x480_MONO);
  misc_info->format=FORMAT_VGA_NONCOMPRESSED;
  misc_info->mode=MODE_640x480_MONO;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f1_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_1);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_800x600_YUV422);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_1;
  misc_info->mode=MODE_800x600_YUV422;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f1_mode1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_1);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_800x600_RGB);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_1;
  misc_info->mode=MODE_800x600_RGB;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f1_mode2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_1);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_800x600_MONO);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_1;
  misc_info->mode=MODE_800x600_MONO;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f1_mode3_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_1);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1024x768_YUV422);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_1;
  misc_info->mode=MODE_1024x768_YUV422;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f1_mode4_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_1);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1024x768_RGB);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_1;
  misc_info->mode=MODE_1024x768_RGB;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f1_mode5_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_1);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1024x768_MONO);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_1;
  misc_info->mode=MODE_1024x768_MONO;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f2_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_2);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1280x960_YUV422);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_2;
  misc_info->mode=MODE_1280x960_YUV422;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f2_mode1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_2);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1280x960_RGB);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_2;
  misc_info->mode=MODE_1280x960_RGB;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f2_mode2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_2);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1280x960_MONO);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_2;
  misc_info->mode=MODE_1280x960_MONO;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f2_mode3_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_2);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1600x1200_YUV422);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_2;
  misc_info->mode=MODE_1600x1200_YUV422;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f2_mode4_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_2);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1600x1200_RGB);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_2;
  misc_info->mode=MODE_1600x1200_RGB;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f2_mode5_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_2);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1600x1200_MONO);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_2;
  misc_info->mode=MODE_1600x1200_MONO;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f6_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_STILL_IMAGE);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_EXIF);
  misc_info->format=FORMAT_STILL_IMAGE;
  misc_info->mode=MODE_EXIF;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f7_mode0_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SCALABLE_IMAGE_SIZE);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_FORMAT7_0);
  misc_info->format=FORMAT_SCALABLE_IMAGE_SIZE;
  misc_info->mode=MODE_FORMAT7_0;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f7_mode1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SCALABLE_IMAGE_SIZE);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_FORMAT7_1);
  misc_info->format=FORMAT_SCALABLE_IMAGE_SIZE;
  misc_info->mode=MODE_FORMAT7_1;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f7_mode2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SCALABLE_IMAGE_SIZE);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_FORMAT7_2);
  misc_info->format=FORMAT_SCALABLE_IMAGE_SIZE;
  misc_info->mode=MODE_FORMAT7_2;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f7_mode3_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SCALABLE_IMAGE_SIZE);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_FORMAT7_3);
  misc_info->format=FORMAT_SCALABLE_IMAGE_SIZE;
  misc_info->mode=MODE_FORMAT7_3;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f7_mode4_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SCALABLE_IMAGE_SIZE);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_FORMAT7_4);
  misc_info->format=FORMAT_SCALABLE_IMAGE_SIZE;
  misc_info->mode=MODE_FORMAT7_4;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f7_mode5_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SCALABLE_IMAGE_SIZE);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_FORMAT7_5);
  misc_info->format=FORMAT_SCALABLE_IMAGE_SIZE;
  misc_info->mode=MODE_FORMAT7_5;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f7_mode6_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SCALABLE_IMAGE_SIZE);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_FORMAT7_6);
  misc_info->format=FORMAT_SCALABLE_IMAGE_SIZE;
  misc_info->mode=MODE_FORMAT7_6;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f7_mode7_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SCALABLE_IMAGE_SIZE);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_FORMAT7_7);
  misc_info->format=FORMAT_SCALABLE_IMAGE_SIZE;
  misc_info->mode=MODE_FORMAT7_7;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}

void
on_fps_menu_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  // function intentionnaly left almost blank: intermediate menu
}


void
on_fps_activate                    (GtkMenuItem     *menuitem,
				    gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_framerate(camera->handle, camera->id, (int)user_data+FRAMERATE_MIN);
  misc_info->framerate=(int)user_data;
  IsoFlowResume();
}


void
on_power_on_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_camera_on(camera->handle, camera->id);
}


void
on_power_off_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_camera_off(camera->handle, camera->id);
}


void
on_power_reset_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_init_camera(camera->handle, camera->id);
}

void
on_trigger_polarity_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_set_trigger_polarity(camera->handle,camera->id,togglebutton->active);
  feature_set->feature[FEATURE_TRIGGER-FEATURE_MIN].trigger_polarity=(int)togglebutton->active;
}


void
on_pan_op_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button, FEATURE_PAN);
}


void
on_pan_power_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_PAN);
}


void
on_tilt_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_TILT);
}


void
on_tilt_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_TILT);
}


void
on_zoom_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_ZOOM);
}


void
on_zoom_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_ZOOM);
}


void
on_focus_op_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_FOCUS);
}


void
on_focus_power_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_FOCUS);
}


void
on_brightness_op_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_BRIGHTNESS);
}


void
on_brightness_power_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_BRIGHTNESS);
}


void
on_gamma_op_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_GAMMA);
}


void
on_gamma_power_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_GAMMA);
}


void
on_saturation_op_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_SATURATION);
}


void
on_saturation_power_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_SATURATION);
}


void
on_hue_op_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_HUE);
}


void
on_hue_power_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_HUE);
}


void
on_whitebal_power_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_WHITE_BALANCE);
}


void
on_whitebal_op_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_WHITE_BALANCE);
}


void
on_sharpness_op_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_SHARPNESS);
}


void
on_sharpness_power_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_SHARPNESS);
}


void
on_exposure_power_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_EXPOSURE);
}


void
on_exposure_op_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_EXPOSURE);
}


void
on_iris_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_IRIS);
}


void
on_iris_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_IRIS);
}


void
on_shutter_power_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_SHUTTER);
}


void
on_shutter_op_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_SHUTTER);
}


void
on_gain_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_GAIN);
}


void
on_gain_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_GAIN);
}


void
on_filter_power_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_OPTICAL_FILTER);
}


void
on_filter_op_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_OPTICAL_FILTER);
}


void
on_pan_man_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_PAN);
}


void
on_tilt_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_TILT);
}


void
on_zoom_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_ZOOM);
}


void
on_focus_man_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_FOCUS);
}


void
on_brightness_man_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_BRIGHTNESS);
}


void
on_gamma_man_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_GAMMA);
}


void
on_saturation_man_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_SATURATION);
}


void
on_hue_man_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_HUE);
}


void
on_whitebal_man_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_WHITE_BALANCE);
}


void
on_sharpness_man_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_SHARPNESS);
}


void
on_exposure_man_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_EXPOSURE);
}


void
on_iris_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_IRIS);
}


void
on_shutter_man_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_SHUTTER);
}


void
on_gain_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_GAIN);
}


void
on_filter_man_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_OPTICAL_FILTER);
}


void
on_trigger_mode_activate              (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  int err;
  err=dc1394_set_trigger_mode(camera->handle, camera->id, (int)user_data);
  feature_set->feature[FEATURE_TRIGGER-FEATURE_MIN].trigger_mode=(int)user_data;
  //UpdateTriggerFrame();
}

void
on_trigger_external_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  int err;
  err=dc1394_feature_on_off(camera->handle, camera->id, FEATURE_TRIGGER, togglebutton->active);
  //printf("trigger error=%d\n",err);
  feature_set->feature[FEATURE_TRIGGER-FEATURE_MIN].is_on=togglebutton->active;
  UpdateTriggerFrame();
}

void
on_memory_channel_activate              (GtkMenuItem     *menuitem,
					 gpointer         user_data)
{
  misc_info->save_channel=(int)user_data; // user data is an int.
  misc_info->load_channel=(int)user_data; // user data is an int.
  UpdateMemoryFrame();
}


void
on_status_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (status_window);
}


void
on_capture_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (capture_window);
}


void
on_capqual_man_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_CAPTURE_QUALITY);
}


void
on_capqual_op_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_CAPTURE_QUALITY);
}


void
on_capqual_power_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_CAPTURE_QUALITY);
}


void
on_capsize_man_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_CAPTURE_SIZE);
}


void
on_capsize_op_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_CAPTURE_SIZE);
}


void
on_capsize_power_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_CAPTURE_SIZE);
}

void
on_capture_start_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dialog = create_capture_multi_dialog();
    gtk_widget_show(dialog);
}


void
on_capture_stop_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_idle_remove(gCaptureIdleID);
  capture_multi_stop();
  gtk_widget_set_sensitive( GTK_WIDGET(lookup_widget( GTK_WIDGET(button), "capture_start")), TRUE);
  gtk_widget_set_sensitive( GTK_WIDGET(lookup_widget( GTK_WIDGET(button), "capture_stop")), FALSE);
  gtk_widget_set_sensitive( GTK_WIDGET(lookup_widget( GTK_WIDGET(button), "capture_single")), TRUE);
}


void
on_capture_single_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  if (capture_single_frame()) {
    GtkWidget *dialog = create_save_single_dialog();
    gtk_widget_show(dialog);
  }
}

void
on_temp_power_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  PowerRangeProcedure(togglebutton,FEATURE_TEMPERATURE);
}


void
on_temp_man_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  ManRangeProcedure(togglebutton,FEATURE_TEMPERATURE);
}


void
on_temp_op_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  OpRangeProcedure(button,FEATURE_TEMPERATURE);
}


void
on_temp_window_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (temperature_window);
}


void
on_set_all_auto_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  // TODO:
  // for each feature, set to auto if available, else set to OP/auto, else do nothing
}


void
on_set_all_man_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  // TODO:
  // for each feature, set to man if available, else do nothing
}


void
on_lock_setup_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  // TODO:
  // pressed: set all to man, then lock all controls
  // depressed: unlock all available controls
}

void
on_load_mem_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  int err;
  //int temp;
  err=dc1394_memory_load(camera->handle,camera->id, misc_info->load_channel);
  //err=dc1394_get_memory_load_ch(camera->handle,camera->id, &temp);
  UpdateAllWindows();

}

void
on_save_mem_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{ 
  int err, timeout;
  dc1394bool_t boolean;
  err=dc1394_set_memory_save_ch(camera->handle,camera->id, misc_info->save_channel);
  err=dc1394_memory_save(camera->handle,camera->id);
  err=dc1394_get_memory_save_ch(camera->handle,camera->id,&timeout);
  boolean=TRUE;
  timeout=LOOP_RETRIES;
  while(boolean & (timeout>0))
    { // wait for save to be completed:
      usleep(LOOP_SLEEP);
      err=dc1394_is_memory_save_in_operation(camera->handle,camera->id, &boolean);
      timeout--;
    }
  
}

void
on_iso_start_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  int err, channel, speed;
  GtkWidget *window = lookup_widget(porthole_window, "camera_scope");

  // future addition: if no iso setup made, do it first.
  err=dc1394_get_iso_channel_and_speed(camera->handle, camera->id, &channel, &speed); //??
  err=dc1394_start_iso_transmission(camera->handle,camera->id);
  misc_info->is_iso_on=DC1394_TRUE;
  gtk_widget_set_sensitive( GTK_WIDGET(lookup_widget( GTK_WIDGET(button), "capture_start")), TRUE);
  gtk_widget_set_sensitive( GTK_WIDGET(lookup_widget( GTK_WIDGET(button), "capture_single")), TRUE);
  //printf("err = %d\n",err);
  if (pi.is_open && (pi.handle == NULL) && (IsoStartThread(window)>0) )
    gIdleID = gtk_idle_add( porthole_idler, NULL);
}


void
on_iso_stop_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  int err;

  //printf("Stopping...\n");
  if ( (pi.is_open = (pi.handle != NULL)) ) {
    gtk_idle_remove(gIdleID);
    IsoStopThread();
  }
  err=dc1394_stop_iso_transmission(camera->handle,camera->id);
  misc_info->is_iso_on=DC1394_FALSE;
  gtk_widget_set_sensitive( lookup_widget( GTK_WIDGET(button), "capture_start"), FALSE);
  gtk_widget_set_sensitive( lookup_widget( GTK_WIDGET(button), "capture_stop"), FALSE);
  gtk_widget_set_sensitive( lookup_widget( GTK_WIDGET(button), "capture_single"), FALSE);
  //printf("... Stopped: err= %d\n",err);
}


void
on_iso_restart_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  int err, channel, speed;
  GtkWidget *window = lookup_widget(porthole_window, "camera_scope");

  if ( (pi.is_open = (pi.handle != NULL)) ) IsoStopThread();
  err=dc1394_stop_iso_transmission(camera->handle,camera->id);
  // future addition: if no iso setup made, do it first.
  err=dc1394_get_iso_channel_and_speed(camera->handle, camera->id, &channel, &speed);//??
  err=dc1394_start_iso_transmission(camera->handle,camera->id);
  misc_info->is_iso_on=DC1394_TRUE;
  if (pi.is_open) IsoStartThread(window);
}


void
on_camera_select_activate              (GtkMenuItem     *menuitem,
					 gpointer         user_data)
{
  GtkWidget *scope = lookup_widget(porthole_window, "camera_scope");
  // set current camera pointers: 
  current_camera=(int)user_data;

  camera=&cameras[current_camera];
  feature_set=&feature_sets[current_camera];
  misc_info=&misc_infos[current_camera];
  capture=&captures[current_camera];
  format7_info=&format7_infos[current_camera];

  //gtk_idle_remove(gIdleID);
  /*IsoStopThread();
  pi.handle=NULL;

  if (pi.handle == NULL)
    // start a separate thread for iso receive
    if (IsoStartThread(scope) > 0)
      // run a gtk idler thread to update the display
  */
  //  gIdleID = gtk_idle_add( porthole_idler, NULL);
  
  // redraw all:
  BuildAllWindows();
  UpdateAllWindows();
}

void
on_porthole_window_size_request        (GtkWidget       *widget,
                                        GtkRequisition  *requisition,
                                        gpointer         user_data)
{
}


void
on_porthole_window_size_allocate       (GtkWidget       *widget,
                                        GtkAllocation   *allocation,
                                        gpointer         user_data)
{

// printf("x: %d, y: %d\n", allocation->width, allocation->height);

  GtkWidget *scope_frame = lookup_widget( widget, "scrolledwindow1");
  gtk_widget_set_usize( scope_frame, allocation->width-200, allocation->height-240);

}


void
on_edit_format7_mode_activate             (GtkMenuItem     *menuitem,
					 gpointer         user_data)
{
  //TODO
  //format7_info->edit_mode=...;
}

void
on_edit_format7_color_activate             (GtkMenuItem     *menuitem,
					 gpointer         user_data)
{
  //TODO
}

void
on_f0_mode6_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_VGA_NONCOMPRESSED);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_640x480_MONO16);
  misc_info->format=FORMAT_VGA_NONCOMPRESSED;
  misc_info->mode=MODE_640x480_MONO16;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f1_mode6_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_1);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_800x600_MONO16);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_1;
  misc_info->mode=MODE_800x600_MONO16;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f1_mode7_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_1);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1024x768_MONO16);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_1;
  misc_info->mode=MODE_1024x768_MONO16;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f2_mode6_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_2);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1280x960_MONO16);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_2;
  misc_info->mode=MODE_1280x960_MONO16;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_f2_mode7_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  int err;
  IsoFlowCheck();
  err=dc1394_set_video_format(camera->handle,camera->id,FORMAT_SVGA_NONCOMPRESSED_2);
  err=dc1394_set_video_mode(camera->handle,camera->id,MODE_1600x1200_MONO16);
  misc_info->format=FORMAT_SVGA_NONCOMPRESSED_2;
  misc_info->mode=MODE_1600x1200_MONO16;
  BuildFpsMenu();
  UpdateTriggerFrame();
  IsoFlowResume();
}


void
on_format7_window_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (format7_window);
}

void
on_scale_value_changed             ( GtkAdjustment    *adj,
				     gpointer         user_data)
{
  int err;

  switch((int)user_data)
    {
      case FEATURE_TEMPERATURE:
	err=dc1394_set_temperature(camera->handle,camera->id,adj->value);
	feature_set->feature[FEATURE_TEMPERATURE-FEATURE_MIN].target_value=adj->value; break;
      case FEATURE_WHITE_BALANCE+BU*4: // why oh why is there a *4?
	err=dc1394_set_white_balance(camera->handle,camera->id,adj->value, feature_set->feature[FEATURE_WHITE_BALANCE-FEATURE_MIN].RV_value);
	feature_set->feature[FEATURE_WHITE_BALANCE-FEATURE_MIN].BU_value=adj->value; break;
      case FEATURE_WHITE_BALANCE+RV*4: // why oh why is there a *4?
	err=dc1394_set_white_balance(camera->handle,camera->id,feature_set->feature[FEATURE_WHITE_BALANCE-FEATURE_MIN].BU_value, adj->value);
	feature_set->feature[FEATURE_WHITE_BALANCE-FEATURE_MIN].RV_value=adj->value; break;
      default: // includes trigger_count
	err=dc1394_set_feature_value(camera->handle,camera->id,(int)user_data,adj->value);
	feature_set->feature[(int)user_data-FEATURE_MIN].value=adj->value;
    }
}

void
on_format7_value_changed             ( GtkAdjustment    *adj,
				       gpointer         user_data)
{
  int err;
  GtkAdjustment* adj2;

  //printf("User data: %d\n",(int)(int*)(int)user_data);
  switch((int)user_data)
    {
      //printf("User data: %d\n",(int)user_data);
      case FORMAT7_SIZE_X:
	err=dc1394_set_format7_image_size(camera->handle,camera->id, format7_info->edit_mode,
					  adj->value, format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].size_y);
	format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].size_x=adj->value;

	// adjust the pos_x adjustment so that (size_x+pos_x)<=max_size_x:
	adj2=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(format7_window, "format7_hposition_scale")));
	adj2->upper=format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].max_size_x-adj->value+1;
	gtk_signal_emit_by_name(GTK_OBJECT (adj2), "changed");
	break;

      case FORMAT7_SIZE_Y:
	err=dc1394_set_format7_image_size(camera->handle,camera->id, format7_info->edit_mode,
					  format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].size_x, adj->value);
	format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].size_y=adj->value;
	// adjust the pos_y adjustment so that (size_y+pos_y)<=max_size_y:
	adj2=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(format7_window, "format7_vposition_scale")));
	adj2->upper=format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].max_size_y-adj->value+1;
	gtk_signal_emit_by_name(GTK_OBJECT (adj2), "changed");
	break;

      case FORMAT7_POS_X:
	err=dc1394_set_format7_image_position(camera->handle,camera->id, format7_info->edit_mode,
					      adj->value, format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].pos_y);
	format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].pos_x=adj->value;
	// adjust the size_x adjustment so that (size_x+pos_x)<=max_size_x:
	adj2=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(format7_window, "format7_hsize_scale")));
	adj2->upper=format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].max_size_x-adj->value+1;
	gtk_signal_emit_by_name(GTK_OBJECT (adj2), "changed");
	break;

      case FORMAT7_POS_Y:
	err=dc1394_set_format7_image_position(camera->handle,camera->id, format7_info->edit_mode,
					      format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].pos_x, adj->value);
	format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].pos_y=adj->value;
	// adjust the size_y adjustment so that (size_y+pos_y)<=max_size_y:
	adj2=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(format7_window, "format7_vsize_scale")));
	adj2->upper=format7_info->mode[format7_info->edit_mode-MODE_FORMAT7_MIN].max_size_y-adj->value+1;
	gtk_signal_emit_by_name(GTK_OBJECT (adj2), "changed");
	break;

      default:
	printf("Error: bad FORMAT7 scale ID passed to 'on_format7_value_changed'\n");
	break;
    }
}

void
on_preferences_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_format6_window_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


gboolean
on_save_single_dialog_delete_event     (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    GtkWidget *dialog = gtk_widget_get_toplevel(widget);
    gtk_widget_destroy(dialog);

    return FALSE;
}


void
on_save_single_dialog_ok_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
    gchar *filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog));
    gtk_widget_hide(dialog);
    save_single_frame(filename);
    gtk_widget_destroy(dialog);
}


void
on_save_single_dialog_cancel_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
    gtk_widget_destroy(dialog);
}


gboolean
on_capture_multi_dialog_delete_event   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    GtkWidget *dialog = gtk_widget_get_toplevel(widget);
    gtk_widget_destroy(dialog);

     return FALSE;
}


void
on_capture_multi_dialog_ok_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
    gchar *filename;
    GtkWidget *window = lookup_widget(capture_window, "capture_window");
    GtkWidget *dialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog));
    gtk_widget_hide(dialog);
    if (capture_multi_start(filename)) {
      gtk_widget_set_sensitive( GTK_WIDGET(lookup_widget( GTK_WIDGET(window), "capture_start")), FALSE);
      gtk_widget_set_sensitive( GTK_WIDGET(lookup_widget( GTK_WIDGET(window), "capture_stop")), TRUE);
      gtk_widget_set_sensitive( GTK_WIDGET(lookup_widget( GTK_WIDGET(window), "capture_single")), FALSE);
      gCaptureIdleID = gtk_idle_add(capture_idler, NULL);
    }
    gtk_widget_destroy(dialog);
}


void
on_capture_multi_dialog_cancel_clicked (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
    gtk_widget_destroy(dialog);
}


void
on_overlay_button_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

  GtkWidget *scope = lookup_widget(porthole_window, "camera_scope");
  
  overlay_power=togglebutton->active;
  if (overlay_power>0)
    {
      if (pi.handle == NULL)
      // start a separate thread for iso receive
      if (IsoStartThread(scope) > 0)
	// run a gtk idler thread to update the display
	gIdleID = gtk_idle_add( porthole_idler, NULL);
    }
  else
    {
      gtk_idle_remove(gIdleID);
      IsoStopThread();
      pi.is_open = FALSE;
    }
}

