/*
 * Copyright (C) 2000-2001 Damien Douxchamps  <douxchamps@ieee.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "callbacks.h"
#include "support.h"
#include "definitions.h"
#include "tools.h"
#include <libdc1394/dc1394_control.h>

extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *status_window;
extern GtkWidget *aperture_window;
extern GtkWidget *configure_window;
extern GtkWidget *capture_window;
extern GtkWidget *temperature_window;
extern dc1394_camerainfo *camera;
extern dc1394_miscinfo *misc_info;
extern dc1394_feature_set *feature_set;
extern Format7Info *format7_info;
extern char* feature_op_list[NUM_FEATURES];
extern char* feature_list[NUM_FEATURES];
extern char* feature_frame_list[NUM_FEATURES];
extern char* feature_scale_list[NUM_FEATURES];
extern char* feature_man_list[NUM_FEATURES];
extern char* feature_auto_list[NUM_FEATURES];
extern char* feature_power_list[NUM_FEATURES];
extern char* trigger_mode_list[4];
extern char* channel_num_list[16];
extern int camera_num;
extern int current_camera;



void
GetFormat7Capabilities(raw1394handle_t handle, nodeid_t node, Format7Info *info)
{
  int i, f, err;
  quadlet_t value;
  
  err=dc1394_query_supported_modes(handle, node, FORMAT_SCALABLE_IMAGE_SIZE, &value);
  for (i=0,f=MODE_FORMAT7_MIN;f<NUM_MODE_FORMAT7;f++,i++)
    {
      info->mode[i].present= (value & (0x1<<i) );
      if (info->mode[i].present) // check for mode presence before query
	{
	  err=dc1394_query_format7_max_image_size(handle,node,f,&info->mode[i].max_size_y,&info->mode[i].max_size_x);
	  err=dc1394_query_format7_unit_size(handle,node,f,&info->mode[i].step_y,&info->mode[i].step_x);
	  err=dc1394_query_format7_image_position(handle,node,f,&info->mode[i].pos_y,&info->mode[i].pos_x);
	  err=dc1394_query_format7_image_size(handle,node,f,&info->mode[i].size_y,&info->mode[i].size_x);
	  
	  err=dc1394_query_format7_pixel_number(handle,node,f,&info->mode[i].pixnum);
	  err=dc1394_query_format7_byte_per_packet(handle,node,f,&info->mode[i].bpp);
	  err=dc1394_query_format7_packet_para(handle,node,f,&info->mode[i].min_bpp,&info->mode[i].max_bpp);
	  err=dc1394_query_format7_total_bytes(handle,node,f,&info->mode[i].total_bytes);
	  
	  // TODO: get color coding id
	  err=dc1394_query_format7_color_coding(handle,node,f,&info->mode[i].color_coding);
	  
	}
    }
  info->edit_mode = MODE_FORMAT7_MIN;

}
